#  TimeStamp  (dos) : 22 Novembre 04 16:57  --  d:\tmp\statgh.r
#
# +----------------------------------------------- (gH) --+
# |                                                       |
# |    statgh.r : quelques fonctions utiles               |
# |               en Rstat pour les statistiques          |
# |               descriptives, les lois et les tests.    |
# |                                                       |
# +-------------------------------------------------------+
#
# on charge ce fichier en crivant
#
#            source("statgh.r)
#
# avec ventuellement un chemin d'accs pour le fichier,
# par exemple :
#
#        - sous Dos
#            source("D:\\RstatDemo\\statgh.r")
#         ou source("D:/RstatDemo/statgh.r")
#
#        - sous Linux
#            source("/home/gh/RstatDemo/statgh.r")
#
#

##############################################################
#
# -  faire : variables locales
#             dtection de la localisation de statgh.r
#             coloration syntaxique (kedit, sedit)
#
##############################################################

# Fonctions dfinies :

#     triAplat
#     triCroise
#     triCroiseAvecMarges
#     decritQT
#     allQT
#     histProba
#     approxPoiss
#     compPourc
#     compMoyData
#     compMoyNoData

# Syntaxe  de ces fonctions :

#     triAplat(titreQL,nomVar,labelMod)
#     triCroise(titreQL1,nomVar1,labelMod1,titreQL2,nomVar2,labelMod2)
#     triCroiseAvecMarges(titreQL1,nomVar1,labelMod1,titreQL2,nomVar2,labelMod2)
#     decritQT(titreQT,nomVar,unite)
#     histProbas(probas,valeurs,titre) ;
#     histEffectifs(effectifs,valeurs,titre) ;
#     approxPoiss(lambda,nbpoints,nbclasses)
#     compPourc(ia,na,ib,nb)
#     compMoyData(var1,var2)
#     compMoyNoData(nb1,moy1,var1,nb2,moy2,var2)

# Exemples d'utilisation de ces fonctions :

#     triAplat("Sexe de la personne",sexeElf,  c("homme","femme") )
#
#     triCroise(
#        "Connaissez-vous les produits bio ?",
#        connaitrePbio,
#        c("non rponse","oui","non"),
#        "Diffrenciez-vous bio et dittique ?",
#        diffPbioPdiet,
#        c("non rponse","oui","non")
#     ) ; " fin de triCroise
#
#   decritQT("nombre d'lves intgrs en 1997",integr,"personne(s)") ;
#
#   allQT(ecoles,c("INT","NPP","NEC","EFT","BUD","DAS","NPM","NOI","MSE")) ;
#
#   histProba( dbinom((0:5),5,0.5) ,c(0:5),"binomiale B(5,0.5)")
#
#   approxPoiss(10,300,15)
#
#   compPourc(20,102,28,98)
#   compMoyData( c(1:8), c(2:10) )
#   compMoyNoData( 97, 5.851, 3.978, 73, 4.877, 5.642 )

# Options modifies :

options(width=180) ;

################################################################

# pour connaitre la version

statgh <- function() {
  cat("\n statgh.r, version 1.31 \n\n")
} ; # fin fonction moy

################################################################

# un simple synonyme pour mean

moy <- function(vecteurQt) {
  return( mean( vecteurQt ) )
} ; # fin fonction moy

################################################################

# la "vraie" variance mathmatique, "biaise"

vvar <- function(vecteurQt) {
  return( mean(vecteurQt*vecteurQt) - mean(vecteurQt)^2 )
} ; # fin fonction vvar

################################################################

# le "vrai" cart-type, issu de la "vraie" variance, mathmatique et "biaise"

ect <- function(vecteurQt) {
  return( sqrt( vvar(vecteurQt) ) )
} ; # fin fonction vvar

################################################################

triAplat <- function(titreQL,nomVar,labelMod) {

################################################################

   cat("\n QUESTION : ",titreQL,"\n\n") ;

   as.vector(table(nomVar))       ->  vetcp    ;  # effectifs
   length(vetcp)                  ->  nbmod    ;  # nombre de valeurs
   sum(vetcp)                     ->  efftot   ;  # effectif total
   round(100*vetcp/efftot)        ->  vftcp    ;  # frquences en %

   matrix(nrow=2,ncol=nbmod+1)    ->  mrtcp   ;

   mrtcp[1,(1:nbmod)]            <- vetcp        ;
   mrtcp[1,  nbmod+1]            <- efftot       ;
   mrtcp[2,(1:nbmod)]            <- vftcp        ;
   mrtcp[2,  nbmod+1]            <- sum(vftcp)   ;


   colnames(mrtcp) <-  c(labelMod," Total") ;
   rownames(mrtcp) <-  c("  Effectif ","  Frquence (en %)")   ;

   print.matrix(mrtcp,quote=FALSE,right=TRUE) ;
   cat("\n") ;

} # fin de fonction triAplat

################################################################

triCroise <- function(titreQL1,nomVar1,labelMod1,titreQL2,nomVar2,labelMod2) {

################################################################

   cat("\n TRI CROISE DES QUESTIONS : \n     ",titreQL1," (en ligne) \n     ",titreQL2," (en colonne)\n") ;
   cat("Effectifs\n") ;

   table(nomVar1,nomVar2)                  ->  mtcp    ;

   rownames(mtcp) <-  labelMod1 ;
   colnames(mtcp) <-  labelMod2 ;

   print.matrix(mtcp,quote=FALSE,right=TRUE) ;

   cat("\n% du total\n") ;
   mtcp <- round(100*mtcp/sum(mtcp))    ;
   print.matrix(mtcp,quote=FALSE,right=TRUE) ;

}# fin de fonction triCroise

################################################################

triCroiseAvecMarges <- function(titreQL1,nomVar1,labelMod1,titreQL2,nomVar2,labelMod2) {

################################################################

   cat("\n TRI CROISE DES QUESTIONS : \n     ",titreQL1," (en ligne) \n     ",titreQL2," (en colonne)\n") ;
   cat("Effectifs\n") ;

   mtcp     <- table(nomVar1,nomVar2) ;
   md       <- dim(mtcp) ;
   gmd      <- dim(mtcp) + 1  ;
   gmtcp    <- matrix(nrow=gmd[1],ncol=gmd[2]) ;

   gmtcp[1:md[1],1:md[2]] <- mtcp ;
   gmtcp[gmd[1] ,1:md[2]] <- table(nomVar2) ;
   gmtcp[1:md[1], gmd[2]] <- table(nomVar1) ;
   tg <- sum(mtcp) ;
   gmtcp[gmd[1] , gmd[2]] <- tg ;

   rownames(gmtcp) <-  c(labelMod1,"TOTAL") ;
   colnames(gmtcp) <-  c(labelMod2,"TOTAL") ;

   print.matrix(gmtcp,quote=FALSE,right=TRUE) ;

   cat("\n% du total\n") ;
   mtcp <- round(100*mtcp/tg)    ;
   gmtcp[1:md[1],1:md[2]] <- mtcp ;
   gmtcp[gmd[1] ,1:md[2]] <- round(100*table(nomVar2)/tg) ;
   gmtcp[1:md[1], gmd[2]] <- round(100*table(nomVar1)/tg) ;
   gmtcp[gmd[1] , gmd[2]] <- 100 ;

   print.matrix(gmtcp,quote=FALSE,right=TRUE) ;

}# fin de fonction triCroiseAvecMarges

################################################################

decritQT <- function(titreQT,nomVar,unite) {

################################################################

 cat("VARIABLE ",titreQT,"\n") ;

 mdsc <- matrix(nrow=6,ncol=2)
 colnames(mdsc) <- rep(" ",2) ;
 rownames(mdsc) <- c(" Taille "," Moyenne"," Ecart-type"," Coef. de variation"," Minimum"," Maximum")

 taille    <- length(nomVar) ;
 moyenne   <- sum(nomVar)/taille ;
 ecartype  <- sqrt( sum(nomVar*nomVar)/taille - moyenne*moyenne ) ;
 cdv       <- round(100*ecartype/moyenne) ;

 mdsc[1,1] <- formatC(taille,format="d",width=4) ;
 mdsc[1,2] <- "individus ";
 mdsc[2,1] <- formatC(moyenne,format="f",width=9,dig=3) ;
 mdsc[2,2] <- unite ;
 mdsc[3,1] <- formatC(ecartype,format="f",width=9,dig=3) ;
 mdsc[3,2] <- unite ;
 mdsc[4,1] <- cdv
 mdsc[4,2] <- "% " ;
 mdsc[5,1] <- min(nomVar) ;
 mdsc[5,2] <- unite;
 mdsc[6,1] <- max(nomVar) ;
 mdsc[6,2] <- unite;

 print.matrix(mdsc,quote=FALSE,right=TRUE) ;

}# fin de fonction decritQT


################################################################

allQT <- function(dataMatrix,nomV) {

################################################################

dm      <-  dim(dataMatrix) ;
nl      <-  dm[1]
nc      <-  dm[2]
resMat  <-  matrix(nrow=nc,ncol=8) ;

# matrice des rsultats

rownames(resMat) <- rep(" ",nc) ;
colnames(resMat) <- c(" Nom "," Num "," Taille "," Moyenne"," Ecart-type"," Coef. de variation"," Minimum"," Maximum")

for (i in 1:nc) {
  v           <- dataMatrix[,i]
  resMat[i,1] <- i
  resMat[i,2] <- nomV[i]
  resMat[i,3] <- length(v)
  resMat[i,4] <- formatC(mean(v),"f",width=9,dig=3)
  resMat[i,5] <- formatC( sqrt(mean(v*v)-mean(v)*mean(v)),"f",width=9,dig=3)
  resMat[i,6] <- formatC( (100*sd(v)/mean(v)),"f",width=5,dig=1)
  resMat[i,7] <- min(v)
  # formatage ventuel
  resMat[i,7] <- formatC(round(1000*as.numeric(resMat[i,7]))/1000,"f",width=9,dig=3)
  resMat[i,8] <- max(v)
  resMat[i,8] <- formatC(round(1000*as.numeric(resMat[i,8]))/1000,"f",width=9,dig=3)
} # fin pour i

# tri par cdv dcroissant

cdv    <- as.numeric(resMat[,6])
cdv    <- cdv * (-1)
odx    <- order(cdv) ;
ordMat <-  matrix(nrow=nc,ncol=8) ;
rownames(ordMat) <- rep(" ",nc) ;
colnames(ordMat) <- c(" Nom "," Num "," Taille "," Moyenne"," Ecart-type"," Coef. de variation"," Minimum"," Maximum")

for (i in 1:nc) {
  ordMat[i,] <- resMat[ odx[i], ]
} # fin pour i

# tri par moyenne dcroissante

moy    <- as.numeric(resMat[,4])
moy    <- moy * (-1)
odx    <- order(moy) ;
moyMat <-  matrix(nrow=nc,ncol=8) ;
rownames(moyMat) <- rep(" ",nc) ;
colnames(moyMat) <- c(" Nom "," Num "," Taille"," Moyenne"," Ecart-type"," Coef. de variation"," Minimum"," Maximum")

for (i in 1:nc) {
  moyMat[i,] <- resMat[ odx[i], ]
} # fin pour i

# affichages

cat("\nDonnes\n") ;
print.matrix(dataMatrix,rowlab=rep(" ",nl),quote=FALSE,right=TRUE) ;

cat("\nPar cdv dcroissant\n") ;
print.matrix(ordMat,quote=FALSE,right=TRUE) ;

cat("\nPar ordre d'entre\n") ;
print.matrix(resMat,quote=FALSE,right=TRUE) ;

cat("\nPar moyenne dcroissante\n") ;
print.matrix(moyMat,quote=FALSE,right=TRUE) ;

# matrice des corrlations

corMat           <-  matrix(nrow=nc,ncol=nc) ;
rownames(corMat) <- as.vector(nomV)
colnames(corMat) <- nomV
meiCor           <- (-3)
iEtj             <- "-99"

for (i in 1:nc) {
  vi <-- dataMatrix[,i]
  mi <-- mean(vi)
  ei <-- sqrt(mean(vi*vi)-mean(vi)*mean(vi))
  for (j in 1:nc) {
      if (i<j) { cor <- "" } else {
         vj  <-- dataMatrix[,j]
         mj  <-- mean(vj)
         ej  <-- sqrt(mean(vj*vj)-mean(vj)*mean(vj))
         cov <- mean(vi*vj) - mi*mj
         cor <- cov/(ei*ej)
         if ((meiCor<cor) & (i!=j)) {
            meiCor <- cor
            iEtj   <- c(i,j)
         } # fin de si sur meiC
         cor <- formatC(cov/(ei*ej),"f",width=5,dig=3)
      } ; # fin de si sur i<j
      corMat[i,j] <- cor
  } # fin pour j
} # fin pour i

cat("\nMatrice des corrlations\n") ;
print.matrix(corMat,quote=FALSE,right=TRUE) ;

# meilleures corrlations

xmeiCor <- iEtj[1]
ymeiCor <- iEtj[2]
nomX    <- nomV[xmeiCor]
nomY    <- nomV[ymeiCor]
lesX    <- dataMatrix[,xmeiCor]
lesY    <- dataMatrix[,ymeiCor]

#lesC <- lm( lesY ~ lesX,model=FALSE )

#  cause des arrondis, on recalcule les valeurs prcises

mX <- mean(lesX)
eX <- sqrt( mean(lesX*lesX) - mX*mX)
mY <- mean(lesY)
eY <- sqrt( mean(lesY*lesY) - mY*mY)
ro <- (mean(lesX*lesY) - mX*mY)/(eX*eY)
vA <- ro*eY/eX
vB <- mY - vA*mX

if (vB<0) { sgn <- "" } else { sgn <- "+" }

cat("\nMeilleure corrlation ",meiCor," pour ",nomX," et ",nomY,"\n") ;
fXY <- paste(nomY," = ",formatC(vA,"f",width=9,dig=3),"*",nomX," ",sgn,formatC(vB,"f",width=9,dig=3)) ;
cat("Formule ",fXY,"\n\n") ;
cat("Tableau des valeurs exactes et approximes : \n") ;

aprMat <- matrix(nrow=nl,ncol=6)
rownames(aprMat) <- rep(" ",nl)
colnames(aprMat) <- c(" Num ",nomX,nomY," recalcul"," diffrence"," som. dif.")

for (i in 1:nbl) {
  aprMat[i,1] <- i
  aprMat[i,2] <- lesX[i]
  # format ventuel des x
  aprMat[i,2] <- formatC(round(1000*as.numeric(aprMat[i,2]))/1000,"f",width=9,dig=3)
  # format ventuel des y
  aprMat[i,3] <- lesY[i]
  aprMat[i,3] <- formatC(round(1000*as.numeric(aprMat[i,3]))/1000,"f",width=9,dig=3)
  aprMat[i,4] <- vA*lesX[i] + vB
  aprMat[i,5] <- formatC(lesY[i] - as.numeric(aprMat[i,4]),"f",width=9,dig=3)
  if (i==1) { aprMat[i,6] <- lesY[i] - as.numeric(aprMat[i,4]) }
  else { aprMat[i,6] <-  as.numeric(aprMat[i-1,6]) + lesY[i] - as.numeric(aprMat[i,4]) }
  aprMat[i,4] <- formatC(round(1000*as.numeric(aprMat[i,4]))/1000,"f",width=9,dig=3)
  aprMat[i,6] <- formatC(round(1000*as.numeric(aprMat[i,6]))/1000,"f",width=9,dig=3)
} # fin pour i

print.matrix(aprMat,quote=FALSE,right=TRUE) ;

# trac correspondant

oX <- order(lesX)
newX <- lesX[ oX]
newY <- lesY[ oX]
par(lwd=2)
postscript("r.ps") ;
plot(newX,newY,"b",col="red",pch=21,bg="orange",fg="black",xlab=nomX,ylab=nomY)
title(paste(" Rgression ",fXY,"   (rho=",meiCor,")"))
abline(c(vB,vA),col="blue")
dev.off()
#graphics.off()

cat("\n Vous pouvez utilisez r.ps\n") ;

}# fin de fonction alltQT


################################################################

histProba <- function(v,pdv,tit) {

################################################################

     lv  <- length(v)  ;
     sv  <- sum(v)  ;
     w  <- 100*v ;
     vx <- pdv ;
     vy <- rep(vx,w) ;
     minvx <- min(vx);
     maxvx <- max(vx);
     xmin <- minvx - 1 ;
     xmax <- maxvx + 1 ;
     brk    <-  c(-0.75+0.5*c(xmin:(2*lv-1)) )
     minay <- min(v)
     if (minay>0) { minay <- 0 }
     maxay <- round(100*max(v))/100
     ldvy <- c(0)
     hy <- 0
     while (hy<maxay) { hy <- hy + 0.1 ; ldvy <- c(ldvy,hy) }
     if (hy>maxay){ maxay <- hy }
     nbax    <- lv + 1
     pdvemin <- min(pdv)-1
     pdvemax <- max(pdv)+1
     brk     <- c(min(pdv) - 0.8 + 0.5*(0:(2*length(pdv))))
     pdve    <- c(pdvemin:pdvemax)
     lpdve   <- c(" ",pdv," ")
     hist(vy,br=brk,col="red", freq = TRUE,xlim=c(xmin,xmax),ylim=100*c(minay,maxay),axes=F,
          main=c("Distribution de la Loi ",tit),xlab="valeurs",ylab="probabilits")
     axis(1,pdve,lpdve,col.axis="blue")
     axis(2,100*ldvy,ldvy,col.axis="blue")


     matH            <-  matrix(nrow=lv,ncol=2)  ;
     matH[,1]        <-  pdv
     matH[,2]        <-  v
     colnames(matH)  <-  c("valeurs","probabilit")
     rownames(matH)  <-  rep(" ",lv)
     print.matrix(matH,quote=FALSE,right=TRUE) ;

} ; # fin fonction histProba

################################################################

histEffectifs <- function(e,v,tit) {

################################################################

    lv      <- length(v) ;
    vmin    <- min(v)
    vmax    <- max(v)
    vy      <- rep(v,e) ;
    pdv     <- v
    brk     <- c(vmin - 0.75 + 0.5*(0:(2*vmax+2)))
    xmin    <- vmin - 1 ;
    xmax    <- vmax + 1 ;
    emin    <- min(e) ;
    emax    <- max(e) ;
    ymin    <- 0 ;
    nbvaly  <- 4

    ech     <- 10**trunc(log10(emax))
    ymax    <- round(emax/ech)
    if (ech*ymax<emax) { ymax <- 0.5 + ymax }
    ymax    <- ech*ymax
    ymax
    largy   <- round(ymax/(4*ech))
    largy
    if (ech*largy<(ymax/4)) { largy <- 0.5 + largy }
    largy
    largy   <- ech*largy
    largy
    valy    <- seq(0,ymax+largy,by=largy)
    valy
    hist(vy,freq =
       TRUE,col="red",br=brk,xlim=c(xmin,xmax),ylim=c(0,ymax+largy),axes=F,
       main=tit,xlab="",ylab="") ;
    #   main=c("Histogramme",tit),xlab="valeurs",ylab="effectifs") ;
    axis(1,c(xmin:xmax),c(" ",v," "),col.axis="blue")
    axis(2,valy,valy,col.axis="blue")

} ; # fin fonction histEffectifs

################################################################

approxPoiss <- function(lambda,nbpoints,nbcla) {

################################################################
par(mfrow=c(1, 2))
tirage    <-  rpois(nbpoints,lambda)

flambda   <- formatC(lambda,format="d")
slambda   <- formatC(sqrt(lambda),format="f",digits=2)
hist(tirage, col="blue", main=paste("Histogramme des frquences"),
     xlab=" valeurs seules",ylab="effectifs",breaks=nbcla)
#str(hist(tirage, col="blue",main=paste("Approximation de la loi de Poisson par la loi normale, lambda=",lambda),nclass=nbclass))

vxp        <-  lambda-3*sqrt(lambda)
vxg        <-  lambda+3*sqrt(lambda)
vx         <-  seq(vxp,vxg,length=nbpoints)
vy         <-  dnorm(vx,mean=lambda-2,sd=sqrt(lambda))

xmin <- min(vx)
xmax <- max(vx)

ymin <- min(vy)
ymax <- max(vy)

hist(tirage, col="blue",
     main=paste("Approximation de la loi de Poisson par la loi normale, lambda=",flambda),breaks=nbcla)
     xlab=paste(" valeurs et loi normale N( ",flambda,",",slambda," )",ylab="",xlim=c(xmin,xmax),ylim=c(ymin,ymax))
par(new=TRUE)
plot(vx,vy,type="l",col="red",lwd=2,axes=FALSE,xlim=c(xmin,xmax),ylim=c(ymin,ymax))
par(mfrow=c(1, 1))

} ; # fin de fonction  approxPoiss

################################################################

compPourc <- function(ia,na,ib,nb) {

################################################################

 pa  <- ia / na ;
 pb  <- ib / nb ;
 p   <- (ia+ib)/(na+nb)
 dp  <- pa - pb
 df  <- abs(dp)
 q   <- p*(1-p)*(1/na+1/nb)
 r <- 1
 eps <- 0
 r   <- sqrt(q)
 eps <- df/r

 cat("\n COMPARAISON DE POURCENTAGES\n\n") ;
 cat("    population A,  ",formatC(ia,width=4)," individus marqus sur ",formatC(na,width=4),
     " soit une proportion de ",formatC(pa,format="f",digits=3),"\n")
 cat("    population B,  ",formatC(ib,width=4)," individus marqus sur ",formatC(nb,width=4),
     " soit une proportion de ",formatC(pb,format="f",digits=3),"\n")
 cat("    globalisation, ",formatC(ia+ib,width=4)," individus marqus sur ",formatC(na+nb,width=4),
     " soit une proportion de ",formatC(p,format="f",digits=3),"\n")
 cat("\n") ;
 cat("    cart-rduit : ",formatC(eps,format="f",digits=4),"\n\n") ;
 cat("    au seuil de 5 % soit 1.96, on peut ")
 if (eps<1.96) {
    cat("accepter")
 } else {
    cat("refuser")
 } ; # fin de si
 cat(" l'hypothse d'galit des pourcentages.\n\n") ;


} ; # fin de fonction  compPourc

################################################################

compMoyData <- function( varA, varB ) {

################################################################

 # modification au 23 avril 2004 : utilisation de variance estime
 # et non pas variance exacte (pour suivre sas)

 na <- length(varA)
 ma <- sum(varA)/na
 va <- (sum(varA*varA)/na-ma*ma)*na/(na-1)
 sa <- sqrt(va)
 ca <- round(100*sa/ma)
 wa <- va/na

 nb <- length(varB)
 mb <- sum(varB)/nb
 vb <- (sum(varB*varB)/nb-mb*mb)*nb/(nb-1)
 sb <- sqrt(vb)
 cb <- round(100*sb/mb)
 wb <- vb/nb

 nu <- abs(ma-mb)
 de <- wa+wb
 r  <- sqrt(de)
 delta <- nu/r

 cat("\n COMPARAISON DE MOYENNES (valeurs fournies)\n\n") ;

 cat("    Variable    nbVal      Moyenne    Variance     Ecart-type     Cdv\n") ;
 cat("       A  ",formatC(na,format="d",width=9),
                  formatC(ma,format="f",width=12,digits=3),
                  formatC(va,format="f",width=12,digits=3),
                  formatC(sa,format="f",width=12,digits=3),
                  formatC(ca,format="d",width=9)," %\n") ;
 cat("       B  ",formatC(nb,format="d",width=9),
                  formatC(mb,format="f",width=12,digits=3),
                  formatC(vb,format="f",width=12,digits=3),
                  formatC(sb,format="f",width=12,digits=3),
                  formatC(cb,format="d",width=9)," %\n") ;

 cat("\n")
 cat("    diffrence rduite : ",formatC(delta,format="f",digits=4),"\n\n") ;
 cat("    au seuil de 5 % soit 1.96, on peut ")
 if (delta<1.96) {
    cat("accepter")
 } else {
    cat("refuser")
 } ; # fin de si
 cat(" l'hypothse d'galit des moyennes.\n\n") ;

} ; # fin de function compMoyData

################################################################

compMoyNoData <- function(na,ma,va,nb,mb,vb) {

################################################################

 sa <- sqrt(va)
 ca <- round(100*sa/ma)
 wa <- va/na

 sb <- sqrt(vb)
 cb <- round(100*sb/mb)
 wb <- vb/nb

 nu <- abs(ma-mb)
 de <- wa+wb
 r  <- sqrt(de)
 delta <- nu/r

 cat("\n COMPARAISON DE MOYENNES (valeurs non fournies)\n\n") ;

 cat("    Variable    nbVal      Moyenne    Variance     Ecart-type      Cdv\n") ;
 cat("       A  ",formatC(na,format="d",width=9),
                  formatC(ma,format="f",width=12,digits=3),
                  formatC(va,format="f",width=12,digits=3),
                  formatC(sa,format="f",width=12,digits=3),
                  formatC(ca,format="d",width=9)," %\n") ;
 cat("       B  ",formatC(nb,format="d",width=9),
                  formatC(mb,format="f",width=12,digits=3),
                  formatC(vb,format="f",width=12,digits=3),
                  formatC(sb,format="f",width=12,digits=3),
                  formatC(cb,format="d",width=9)," %\n") ;

 cat("\n")
 cat("    diffrence rduite : ",formatC(delta,format="f",digits=4),"\n\n") ;
 cat("    au seuil de 5 % soit 1.96, on peut ")
 if (delta<1.96) {
    cat("accepter")
 } else {
    cat("refuser")
 } ; # fin de si
 cat(" l'hypothse d'galit des moyennes.\n\n") ;

} ; # fin de function compMoyNoData

#
# fonction copie avec l'autorisation de son auteur
#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Shapefile Format - Read/Write shapefile format within R
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Ben Stabler benjamin.stabler@odot.state.or.us
# Copyright (C) 2003  Oregon Department of Transportation
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#Read DBF format
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

read.dbf <- function(dbf.name) {

        infile<-file(dbf.name,"rb")

        # Header

        file.version <- readBin(infile,integer(), 1, size=1, endian="little")
        file.year <- readBin(infile,integer(), 1, size=1, endian="little")
        file.month <- readBin(infile,integer(), 1, size=1, endian="little")
        file.day <- readBin(infile,integer(), 1, size=1, endian="little")
        num.records <- readBin(infile,integer(), 1, size=4, endian="little")
        header.length <- readBin(infile,integer(), 1, size=2, endian="little")
        record.length <- readBin(infile,integer(), 1, size=2, endian="little")
        file.temp <- readBin(infile,integer(), 20, size=1, endian="little")
        header <- list(file.version,file.year, file.month, file.day, num.records, header.length, record.length)
        names(header) <- c("file.version","file.year","file.month","file.day","num.records","header.length","record.length")
        rm(file.version,file.year, file.month, file.day, num.records, header.length, record.length)

        # Calculate the number of fields

        num.fields <- (header$header.length-32-1)/32
        field.name <- NULL
        field.type <- NULL
        field.length <- NULL
        field.decimal <- NULL

        # Field Descriptions (32 bytes each)

        for (i in 1:num.fields) {
                field.name.test <- readBin(infile,character(), 1, size=10, endian="little")
                field.name <- c(field.name,field.name.test)
                if (nchar(field.name.test)!=10) {
                   file.temp <- readBin(infile,integer(), 10-(nchar(field.name.test)), 1, endian="little")
                } # fin de si
                field.type <- c(field.type,readChar(infile, 1))
                file.temp <- readBin(infile,integer(), 4, 1, endian="little")
                field.length <- c(field.length,readBin(infile,integer(), 1, 1, endian="little"))
                field.decimal <- c(field.decimal, readBin(infile,integer(), 1, 1, endian="little"))
                file.temp <- readBin(infile,integer(), 14, 1, endian="little")
        } # fin de pour i

        # Create a table of the field info

        fields <- data.frame(NAME=field.name,TYPE=field.type,LENGTH=field.length,DECIMAL=field.decimal)

        # Set all fields with length<0 equal to correct number of characters

        fields$LENGTH[fields$LENGTH<0]<-(256+fields$LENGTH[fields$LENGTH<0])

        # Read in end of attribute descriptions terminator - should be integer value 13

        file.temp <- readBin(infile,integer(), 1, 1, endian="little")

        # Increase the length of field 1 by one to account for the space at the beginning of each record

        fields$LENGTH[1]<-fields$LENGTH[1]+1

        # Add fields to the header list

        header <- c(header,fields=NULL)
        header$fields <- fields

        # Read in each record to a list element

        all.records <- list()
        for (i in 1:header$num.records) {
                all.records <- c(all.records, list(readChar(infile, header$record.length)))
        } # fin de pour i

        # Close the dbf file connection

        close(infile)

        #Function to split the strings and replace all " " with "" at the end of string

        format.record <- function(record) {
                record <- substring(record, c(1,cumsum(fields$LENGTH)[1:length(cumsum(fields$LENGTH))-1]+1),cumsum(fields$LENGTH))
                record <- gsub(" +$","", record)
                record
        } # fin de fonction format.record

        # Split each record into columns and save as data.frame

        dbf <- data.frame(t(data.frame(lapply(all.records, format.record))))
        rm(all.records)
        dimnames(dbf) <- list(1:header$num.records, header$fields$NAME)

        # Set the numeric fields to numeric

        for (i in 1:ncol(dbf)) {
                if(fields$TYPE[i]=="C") { dbf[[i]] <- as.character(dbf[[i]]) }
                if(fields$TYPE[i]=="N") { dbf[[i]] <- as.numeric(as.character(dbf[[i]])) }
                if(fields$TYPE[i]=="F") { d
                    bf[[i]] <- as.numeric(as.character(dbf[[i]]))
                    warning("Possible trouble converting numeric field in the DBF\n")
                } # fin de si type F
        } # fin de pour i

        # If the first field is of type character then remove the first
        # character of each record since the DBF stores a space for a
        # valid record and an * for a deleted record.
        # If the field is numeric then R removes the white space

        if(fields[1,2]=="C") { dbf[[1]] <- gsub("^[ *]", "", as.character(dbf[[1]])) }

        colnames(dbf) <- as.character(fields$NAME)
        colnames(dbf) <- gsub("_",".",colnames(dbf))

        # Return the dbf as a list with a data.frame and a header list

        list(dbf=dbf, header=header)

} # fin de fonction read.dbf

################################################################

z <- function() { quit(save="no") }

################################################################

##  pb si 0,2 au lieu de 0.2
##  variable d'environnement pour trouver statgh.r ?
##  tester si fichier existe ? poser des questions ?
## dtection des paramtres ?

cat("\n\n Vous pouvez utiliser les fonctions\n\n")

cat("     triAplat             \n")
cat("     triCroise            \n")
cat("     triCroiseAvecMarges  \n")
cat("     decritQT             \n")
cat("     allQT                \n")
cat("     histProba            \n")
cat("     approxPoiss          \n")
cat("     compPourc            \n")
cat("     compMoyData          \n")
cat("     compMoyNodata        \n\n")

statgh()

## fin de statgh.r
